@@ -55,7 +55,7 @@ class KIpMsgEvent: public IpMessengerEvent {
 		virtual void VersionInfoRecieveAfter( HostListItem &host, string version );
 		virtual void AbsenceDetailRecieveAfter( HostListItem &host, string absenceDetail );
 
-		void ShowSendDlg();
+		SendDialog * ShowSendDlg();
 		void ShowHiddenRecieveMsg();
 		void FindAndShowHiddenRecieveMsg( RecievedMessage& msg );
 
@@ -67,8 +67,8 @@ class KIpMsgEvent: public IpMessengerEvent {
 		QPtrList<SendDialog>& GetSendDialogs();
 		static string CreateHostInfoString(HostListItem host);
 		static void GetHostEncodingFromConfig( HostListItem &host );
-	private:
 		void RefreshHostListInAllSendDlg();
+	private:
 		void ShowRecieveMsg( RecievedMessage& msg );
 		QPtrList<SendDialog> sendDialogs;
 		QPtrList<RecieveDialog> recieveDialogs;
