@@ -87,7 +87,7 @@ AttachFile KIpMsgAttachedFileListViewItem::file()
 QString KIpMsgAttachedFileListViewItem::ExtName( QString FileName )
 {
 	int pos = FileName.findRev( "." );
-	QString ExtName = "";
+	QString ExtName( "" );
 	if ( pos >= 0 ) {
 		ExtName = FileName.mid( pos + 1 );
 	}
@@ -119,11 +119,7 @@ QString KIpMsgAttachedFileListViewItem::FileType( QString FileName )
  */
 QString KIpMsgAttachedFileListViewItem::TimeStamp( time_t t )
 {
-	char buf[100];
-	memset( buf, 0, sizeof( buf ) );
-	ctime_r( &t, buf );
-	buf[strlen( buf ) - 1] = 0;
-	return QString(buf);
+	return CreateTimeString( t );
 }
 
 /**
@@ -403,7 +399,7 @@ void RecieveDialog::slotReplyClicked()
 		}
 		//引用文字を行頭に付加する。(改行文字で分割)
 		QStringList RecvMsg = QStringList::split("\n", m_RecievedMessageHTMLPart->selectedText() );
-		QString quoteStr;
+		QString quoteStr("");
 		QString quoteChar = KIpMsgSettings::quoteChar();
 		for( QStringList::iterator quote = RecvMsg.begin(); quote != RecvMsg.end(); quote++) {
 			quoteStr.append( quoteChar );
@@ -613,8 +609,6 @@ void RecieveDialog::resizeEvent( QResizeEvent *e )
  */
 void RecieveDialog::doResize( QResizeEvent * /*e*/ )
 {
-	char buf[30];
-
 	if ( msg.IsMulticast() ) {
 		m_MessageGroup->setTitle("Multicast from ...");
 	}
@@ -624,10 +618,8 @@ void RecieveDialog::doResize( QResizeEvent * /*e*/ )
 	if ( msg.IsPasswordLock() ) {
 		m_OpenButton->setText(tr2i18n("Open(Password Locked)"));
 	}
-	time_t recv = msg.Recieved();
-	ctime_r( &recv, buf );
-	buf[strlen(buf) - 1] = 0;
-	m_MessageRecievedAtLabel->setText( "at " + QString( buf ) );
+
+	m_MessageRecievedAtLabel->setText( "at " + CreateTimeString( msg.Recieved() ) );
 	QRect rectGroup = m_MessageGroup->geometry();
 
 	QRect rectFrame = m_RecvAreaFrame->geometry();
@@ -682,8 +674,8 @@ QString RecieveDialog::convertMessageToHTML( QString msg )
 						.replace('<', "&lt;")
 						.replace('>',"&gt;")
 						.replace(' ', "&nbsp;");
-	QString url = "";
-	QString returnHtmlBody = "";
+	QString url("");
+	QString returnHtmlBody("");
 	bool isInURLString = false;
 	//Process htmlBody
 	for( unsigned int i = 0; i < htmlBody.length(); i++){
